
#include "pch.h"

#include "CryptoC++.h"
#include "EaUser.h"
#include "MtConverter.h"
#include <time.h>
#include <sstream>
#include <fstream>
#include "UserLic.h"
#include <openssl\evp.h>
#include <openssl\err.h>
#include <openssl\applink.c>
#include <openssl\sha.h>



static const unsigned int KEY_SIZE = 16;
static const unsigned int BLOCK_SIZE = AES_BLOCK_SIZE;


using EVP_CIPHER_CTX_free_ptr = std::unique_ptr<EVP_CIPHER_CTX, decltype(&::EVP_CIPHER_CTX_free)>;

void handleErrors(void);
int aes_decrypt(const byte* key, const byte* iv, const byte* pCtext, byte* pRtext, int iTextsz);
int aes_encrypt(const byte* key, const byte* iv, const byte* pPtext, byte* pCtext, int iTextsz);

std::string AES_NormalizeKey(const void *const apBuffer, size_t aSize)
{
	
	static const unsigned char key32[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 };
	const char *const Buffer = reinterpret_cast<const char *>(apBuffer);
	std::string Result(reinterpret_cast<const char *>(key32), 32);
	int sz = (int)((aSize < 32) ? aSize : 32);
	std::copy(Buffer, Buffer + sz, Result.begin());
	Result.resize(sz);
	return Result;
}
bool SHA256(void* input, unsigned long length, unsigned char* md)
{
	SHA256_CTX context;
	if (!SHA256_Init(&context)) {
		std::cout << "SHA256_Init ";
		handleErrors();
		return false;
	}

	if (!SHA256_Update(&context, (unsigned char*)input, length)) {
		std::cout << "SHA256_Update ";
		handleErrors();
		return false;
	}

	if (!SHA256_Final(md, &context)) {
		std::cout << "SHA256_Final ";
		handleErrors();
		return false;
	}

	return true;
}
int main()
{
	byte ukey[] = "qwertyuiopasdfgh";
	byte iv[BLOCK_SIZE];
	
	std::string lc{ "lic_C#.txt" };
	size_t sz = std::experimental::filesystem::file_size(lc);
	std::ifstream fl(lc, std::ifstream::binary);
	char* ciphertext = new char [sz];
	fl.read(ciphertext, sz);
	fl.close();
	byte* pfl = new byte[sz];
	std::cout << sz;
	int dcount = (int)aes_decrypt(ukey, iv, (const byte* )ciphertext, pfl, (int)sz);
	USERLIC u(pfl);
	u.AddAcc(98765);
	std::cout << u.ToString();
	size_t s = u.GetSize();
	s = (size_t)std::ceill(1 + (long double)s / BLOCK_SIZE) * BLOCK_SIZE;
	byte* prs  = new byte[s];
	byte* ers = new byte[s + BLOCK_SIZE];
	size_t init = u.ToArray(prs);
	size_t res = aes_encrypt(ukey, iv, prs, ers, (int)s);
	std::ofstream fw("lic_++.txt", std::ofstream::binary);
	fw.write((const char*)ers, res);
	fw.close();
	delete[] pfl;
	delete[] prs;
	delete[] ers;
/************************************* SHA256 ************************************/ 
	unsigned char shares[SHA256_DIGEST_LENGTH];
	if (SHA256(ukey, sizeof(ukey), shares)) {
		std::ofstream fw("sha256_++.bin", std::ofstream::binary);
		for (int i = 0; i < sizeof(shares); i++)
			std::cout << "  " << std::hex << (int)shares[i];
		std::cout << std::endl;
		fw.write((const char*)shares, SHA256_DIGEST_LENGTH);
		fw.close();
	}
	return 0;
}

void handleErrors(void)
{
	ERR_print_errors_fp(stderr);
//	abort();
}

int aes_encrypt(const byte* key, const byte* iv, const byte* pPtext, byte* pCtext, int iTextsz)
{
	EVP_CIPHER_CTX_free_ptr ctx(EVP_CIPHER_CTX_new(), ::EVP_CIPHER_CTX_free);

	int rc = EVP_EncryptInit_ex(ctx.get(), EVP_aes_128_ecb(), NULL, key, iv);
	if (rc != 1) {
		std::cout << "EVP_EncryptInit_ex failed ";
		handleErrors();
		return 0;
	}

	EVP_CIPHER_CTX_set_padding(ctx.get(), 0/* EVP_PADDING_ZERO*/);


	int out_len1 = iTextsz + BLOCK_SIZE;

	rc = EVP_EncryptUpdate(ctx.get(), pCtext, &out_len1, pPtext, iTextsz);
	if (rc != 1) {
		std::cout << "EVP_EncryptUpdate failed ";
		handleErrors();
		return 0;
	}
	int out_len2 = iTextsz - out_len1;
	rc = EVP_EncryptFinal_ex(ctx.get(), pCtext + out_len1, &out_len2);
	if (rc != 1) {
		std::cout << "EVP_EncryptFinal_ex failed ";
		handleErrors();
		return 0;
	}

	return out_len1 + out_len2;
}

int aes_decrypt(const byte* key, const byte* iv, const byte* pCtext, byte* pRtext, int iTextsz) {
	
	EVP_CIPHER_CTX_free_ptr ctx(EVP_CIPHER_CTX_new(), ::EVP_CIPHER_CTX_free);

	int rc = EVP_DecryptInit_ex(ctx.get(), EVP_aes_128_ecb(), NULL, key, iv);
	if (rc != 1) {
 		std::cout << "EVP_EncryptInit_ex failed ";
		handleErrors();
		return 0;
	}

	EVP_CIPHER_CTX_set_padding(ctx.get(), 0/* EVP_PADDING_ZERO*/);

	int out_len1 = iTextsz;

	rc = EVP_DecryptUpdate(ctx.get(), pRtext, &out_len1, pCtext, iTextsz);
	if (rc != 1) {
 		std::cout << "EVP_EncryptUpdate failed ";
		handleErrors();
		return 0;
	}


	int out_len2 = iTextsz - out_len1;
	rc = EVP_DecryptFinal_ex(ctx.get(), pRtext + out_len1, &out_len2);
	if (rc != 1) {
 		std::cout << "EVP_EncryptFinal_ex failed ";
		handleErrors();
		return 0;
	}


	return out_len1 + out_len2;

}
